@@ -25,6 +25,7 @@ import neo4j
 import xgt
 import os
 import time
+from enum import Enum
 
 class Neo4jDriver(object):
     def __init__(self, host = 'localhost',
@@ -264,10 +265,27 @@ class Neo4jConnector(object):
         'PointArray': pa.list_(pa.list_(pa.float32())),
     }
 
+    class _Labels(Enum):
+        HAS_BOTH = 0,
+        BOTH_EMPTY = 1,
+        SOURCE_EMPTY = 2,
+        TARGET_EMPTY = 3
+
+        @classmethod
+        def get_label_type(cls, has_source, has_target):
+            if has_source and has_target:
+                return cls.HAS_BOTH
+            elif has_source:
+                return cls.TARGET_EMPTY
+            elif has_target:
+                return cls.SOURCE_EMPTY
+            else:
+                return cls.BOTH_EMPTY
+
     def __init__(self, xgt_server,
                        neo4j_driver,
                        verbose = False,
-                       disable_apoc = False):
+                       enable_apoc = True):
         """
         Initializes the connector class.
 
@@ -279,9 +297,10 @@ class Neo4jConnector(object):
             Connection object to Neo4j.
         verbose : bool
             Print detailed information during calls.
-        disable_apoc : bool
+        enable_apoc : bool
             If the connector finds APOC, it will use that to improve schema queries.
-            If set to True this disables that feature.
+            If set to True this enables that feature.
+            By default this is True.
         """
 
         self._xgt_server = xgt_server
@@ -296,7 +315,7 @@ class Neo4jConnector(object):
         self.__verbose = verbose
         self._default_namespace = xgt_server.get_default_namespace()
 
-        self._neo4j_has_apoc = False if disable_apoc else self.__neo4j_check_for_apoc()
+        self._neo4j_has_apoc = False if not enable_apoc else self.__neo4j_check_for_apoc()
         if self.__verbose and self._neo4j_has_apoc:
             print("Using apoc to query schema.")
 
@@ -421,7 +440,8 @@ class Neo4jConnector(object):
     def get_xgt_schemas(self, vertices = None, edges = None,
                         neo4j_id_name = 'neo4j_id',
                         neo4j_source_node_name = 'neo4j_source',
-                        neo4j_target_node_name = 'neo4j_target') -> dict():
+                        neo4j_target_node_name = 'neo4j_target',
+                        import_edge_nodes = True) -> dict():
         """
         Retrieve a dictionary containing the schema information for all of
         the nodes/vertices and all of the edges requested.
@@ -430,16 +450,23 @@ class Neo4jConnector(object):
         Parameters
         ----------
         vertices : iterable
-            List of requested node labels (vertex frame names).
+            List of requested node labels (vertex frame names) as a string or tuple.
+            To map to a specific from a Neo4j label to a xGT frame use a tuple like so:
+            ('neo4j_name', 'xgt_name').
         edges : iterable
-            List of requested relationship type (edge frame) names.
-            Any vertices not given for an edge will be automatically requested.
+            List of requested relationship type (edge frame) names or tuple.
+            To map to a specific from a Neo4j type to a xGT frame use a tuple like so:
+            ('neo4j_name', 'xgt_name').
+            Any vertices not given for an edge will be automatically requested unless disabled.
         neo4j_id_name : str
             The name of the xGT column holding the Neo4j node's ID value.
         neo4j_source_node_name : str
             The name of the xGT column holding the source node's ID value.
         neo4j_source_node_name: str
             The name of the xGT column holding the target node's ID value.
+        import_edge_nodes : boolean
+            Add vertices from edge if not explicitly listed.
+            By default True.
 
         Returns
         -------
@@ -452,25 +479,42 @@ class Neo4jConnector(object):
         self.__update_cache_state()
 
         if vertices is None and edges is None:
-           vertices = {vertex : None for vertex in self.__neo4j_node_labels(False)}
-           edges = {edge : None for edge in self.__neo4j_relationship_types(False)}
+           vertices = [vertex for vertex in self.__neo4j_node_labels(False)]
+           edges = [edge for edge in self.__neo4j_relationship_types(False)]
         elif edges is None:
-            edges = { }
+            edges = [ ]
         elif vertices is None:
-            vertices = { }
+            vertices = [ ]
+
+        mapping_vertices = { }
+        mapping_edges = { }
+        for val in vertices:
+            if isinstance(val, str):
+                if val != '':
+                    mapping_vertices[val] = val
+                else:
+                    mapping_vertices[val] = self.__xgt_unlabeled_vertex_name()
+            elif isinstance(val, tuple):
+                mapping_vertices[val[0]] = val[1]
+
+        for val in edges:
+            if isinstance(val, str):
+                mapping_edges[val] = val
+            elif isinstance(val, tuple):
+                mapping_edges[val[0]] = val[1]
 
         # Any vertices not given for an edge will be added to vertices.
-        for edge in edges:
+        for edge in mapping_edges:
             if edge not in self.__neo4j_relationship_types(False):
                 raise ValueError(f"Neo4j Relationship {edge} is not found.")
-            schemas = self.__extract_xgt_edge_schemas(edge, vertices,
-                neo4j_source_node_name, neo4j_target_node_name, False)
+            schemas = self.__extract_xgt_edge_schemas(edge, mapping_vertices, mapping_edges,
+                neo4j_source_node_name, neo4j_target_node_name, import_edge_nodes, False)
             result['edges'][edge] = schemas
 
-        for vertex in vertices:
-            if vertex not in self.__neo4j_node_labels(False):
+        for vertex in mapping_vertices:
+            if vertex != self.__neo4j_unlabeled_vertex_name() and vertex not in self.__neo4j_node_labels(False):
                 raise ValueError(f"Neo4j Node Label {vertex} is not found.")
-            table_schema = self.__extract_xgt_vertex_schema(vertex, neo4j_id_name, False)
+            table_schema = self.__extract_xgt_vertex_schema(vertex, neo4j_id_name, mapping_vertices[vertex], False)
             result['vertices'][vertex] = table_schema
             if self.__verbose:
                 print(f"xGT graph schema for vertex {vertex}: {table_schema}")
@@ -509,17 +553,19 @@ class Neo4jConnector(object):
         """
         if not append:
             for edge in xgt_schemas['edges']:
-                self._xgt_server.drop_frame(edge)
                 schemas = xgt_schemas['edges'][edge]
+                if (len(schemas) == 1):
+                    self._xgt_server.drop_frame(schemas[0]['xgt_name'])
                 # Frame name refers to multiple edges:
-                if (len(schemas) > 0):
+                elif (len(schemas) > 1):
                     for schema in schemas:
-                        multi_edge_name = self.__edge_name_transform(edge, schema['source'], schema['target'], True)
+                        multi_edge_name = self.__edge_name_transform(schema['xgt_name'], schema['xgt_source'], schema['xgt_target'], True)
                         self._xgt_server.drop_frame(multi_edge_name)
 
             for vertex in xgt_schemas['vertices']:
+                schema = xgt_schemas['vertices'][vertex]
                 try:
-                    self._xgt_server.drop_frame(vertex)
+                    self._xgt_server.drop_frame(schema['xgt_name'])
                 except xgt.XgtFrameDependencyError as e:
                     if force:
                         # Would be better if this could be done without doing this.
@@ -534,12 +580,13 @@ class Neo4jConnector(object):
             table_schema = schema['schema']
             key = schema['key']
             create_frame = True
+            vname = schema['xgt_name']
             if append:
                 try:
-                    frame = self._xgt_server.get_vertex_frame(vertex)
+                    frame = self._xgt_server.get_vertex_frame(vname)
                     if frame.schema != table_schema:
                         raise ValueError(
-                            f"Vertex Frame {vertex} has a schema {frame.schema}"
+                            f"Vertex Frame {vname} has a schema {frame.schema}"
                             + f" that is incompatible with Neo4j: {table_schema}"
                         )
                     create_frame = False
@@ -547,7 +594,7 @@ class Neo4jConnector(object):
                     pass
             if create_frame:
                 self._xgt_server.create_vertex_frame(
-                    name = vertex, schema = table_schema,
+                    name = vname, schema = table_schema,
                     key = key, attempts = 5,
                 )
 
@@ -557,12 +604,12 @@ class Neo4jConnector(object):
                 table_schema = schema['schema']
                 if self.__verbose:
                     print(f"xGT graph schema for edge {edge}: {table_schema}")
-                name = self.__edge_name_transform(edge, schema['source'], schema['target'], transform)
+                name = self.__edge_name_transform(schema['xgt_name'], schema['xgt_source'], schema['xgt_target'], transform)
                 self._xgt_server.create_edge_frame(
                         name = name, schema = table_schema,
-                        source = schema['source'],
+                        source = schema['xgt_source'],
                         source_key = schema['source_key'],
-                        target = schema['target'],
+                        target = schema['xgt_target'],
                         target_key = schema['target_key'],
                         attempts = 5,
                     )
@@ -596,15 +643,34 @@ class Neo4jConnector(object):
         # Use the count store to get totals.
         estimated_counts = 0
         for vertex in xgt_schemas['vertices']:
+            # On transfer on unlabeled vertices we can't get accurate
+            # estimates so assume every vertex.
+            if vertex == '':
+                q = f"MATCH (v) RETURN count(v)"
+                with self._neo4j_driver.query(q, False) as query:
+                    for record in query.result():
+                        estimated_counts = record[0]
+                break
             q = f"MATCH (v:{vertex}) RETURN count(v)"
             with self._neo4j_driver.query(q, False) as query:
                 for record in query.result():
                     estimated_counts += record[0]
-        for edge in xgt_schemas['edges']:
-            q = f"MATCH ()-[e:{edge}]->() RETURN count(e)"
-            with self._neo4j_driver.query(q, False) as query:
-                for record in query.result():
-                    estimated_counts += record[0]
+        for edge, schema_list in xgt_schemas['edges'].items():
+            for schema in schema_list:
+                source = schema['source']
+                target = schema['target']
+                match_type = schema['empty_labels']
+                # We can't use both source and target, because that won't use
+                # the count store.
+                if match_type == self._Labels.SOURCE_EMPTY:
+                    q = f"MATCH ()-[e:{edge}]->(:{target}) RETURN count(e)"
+                elif match_type == self._Labels.TARGET_EMPTY:
+                    q = f"MATCH (:{source})-[e:{edge}]->() RETURN count(e)"
+                else:
+                    q = f"MATCH ()-[e:{edge}]->() RETURN count(e)"
+                with self._neo4j_driver.query(q, False) as query:
+                    for record in query.result():
+                        estimated_counts += record[0]
 
         with self.progress_display(estimated_counts) as progress_bar:
             for vertex, schema in xgt_schemas['vertices'].items():
@@ -613,24 +679,37 @@ class Neo4jConnector(object):
                 table_schema = schema['schema']
                 attributes = {_:t for _, t, *_unused_ in table_schema}
                 key = schema['key']
-                query = f"MATCH (v:{vertex}) RETURN id(v) AS {key}"  # , {', '.join(attributes)}"
+                if vertex != '':
+                    query = f"MATCH (v:{vertex}) RETURN id(v) AS {key}"  # , {', '.join(attributes)}"
+                else:
+                    query = f"MATCH (v) where size(labels(v)) = 0 RETURN id(v) AS {key}"  # , {', '.join(attributes)}"
                 for a in attributes:
                     if a != key:
                         query += xlate_result_property(a, attributes[a]) # f", v.{a} AS {a}"
-                self.__copy_data(query, vertex, schema['neo4j_schema'], progress_bar)
+                # Is an empty vertex type if None:
+                if schema['neo4j_schema'] is not None:
+                    self.__copy_data(query, schema['xgt_name'], schema['neo4j_schema'], progress_bar)
             for edge, schema_list in xgt_schemas['edges'].items():
                 if self.__verbose:
                     print(f'Copy data for node {edge} into schema: {schema_list}')
                 transform = True if len(schema_list) > 1 else False
                 for schema in schema_list:
-                    name = self.__edge_name_transform(edge, schema['source'], schema['target'], transform)
+                    name = self.__edge_name_transform(schema['xgt_name'], schema['xgt_source'], schema['xgt_target'], transform)
                     table_schema = schema['schema']
                     attributes = {_:t for _, t, *_unused_ in table_schema}
                     source = schema['source']
                     source_key = schema['source_key']
                     target = schema['target']
                     target_key = schema['target_key']
-                    query = f"MATCH (u:{source})-[e:{edge}]->(v:{target}) RETURN"
+                    match_type = schema['empty_labels']
+                    if match_type == self._Labels.HAS_BOTH:
+                        query = f"MATCH (u:{source})-[e:{edge}]->(v:{target}) RETURN"
+                    elif match_type == self._Labels.BOTH_EMPTY:
+                        query = f"MATCH (u)-[e:{edge}]->(v) where size(labels(u)) = 0 and size(labels(v)) = 0 RETURN"
+                    elif match_type == self._Labels.SOURCE_EMPTY:
+                        query = f"MATCH (u)-[e:{edge}]->(v:{target}) where size(labels(u)) = 0 RETURN"
+                    elif match_type == self._Labels.TARGET_EMPTY:
+                        query = f"MATCH (u:{source})-[e:{edge}]->(v) where size(labels(v)) = 0 RETURN"
                     query += f" id(u) AS {source_key}"
                     query += f", id(v) AS {target_key}"
                     for a in attributes:
@@ -643,7 +722,8 @@ class Neo4jConnector(object):
                         neo4j_id_name = 'neo4j_id',
                         neo4j_source_node_name = 'neo4j_source',
                         neo4j_target_node_name = 'neo4j_target',
-                        append = False, force = False) -> None:
+                        append = False, force = False,
+                        import_edge_nodes = True) -> None:
         """
         Copies data from Neo4j to Trovares xGT.
 
@@ -657,10 +737,14 @@ class Neo4jConnector(object):
         Parameters
         ----------
         vertices : iterable
-            List of requested node labels (vertex frame names).
+            List of requested node labels (vertex frame names) as a string or tuple.
+            To map to a specific from a Neo4j label to a xGT frame use a tuple like so:
+            ('neo4j_name', 'xgt_name').
         edges : iterable
-            List of requested relationship type (edge frame) names.
-            Any vertices not given for an edge will be automatically requested.
+            List of requested relationship type (edge frame) names or tuple.
+            To map to a specific from a Neo4j type to a xGT frame use a tuple like so:
+            ('neo4j_name', 'xgt_name').
+            Any vertices not given for an edge will be automatically requested unless disabled.
         neo4j_id_name : str
             The name of the xGT column holding the Neo4j node's ID value.
         neo4j_source_node_name : str
@@ -674,13 +758,17 @@ class Neo4jConnector(object):
             any existing frames with the same names prior to creation).
         force : boolean
             Set to true to force xGT to drop edges when a vertex frame has dependencies.
+        import_edge_nodes : boolean
+            Add vertices from edge if not explicitly listed.
+            By default True.
 
         Returns
         -------
             None
         """
         xgt_schema = self.get_xgt_schemas(vertices, edges,
-                neo4j_id_name, neo4j_source_node_name, neo4j_target_node_name)
+                neo4j_id_name, neo4j_source_node_name, neo4j_target_node_name,
+                import_edge_nodes)
         self.create_xgt_schemas(xgt_schema, append, force)
         self.copy_data_to_xgt(xgt_schema)
         return None
@@ -713,8 +801,6 @@ class Neo4jConnector(object):
         -------
             None
         """
-        if namespace == None:
-            namespace = self._default_namespace
         xgt_server = self._xgt_server
         if namespace == None:
             namespace = self._default_namespace
@@ -913,7 +999,7 @@ class Neo4jConnector(object):
             self._count += count_to_add
             # Counts are no longer accurate
             while (self._count > self._total_count):
-                self._total_count *= 2
+                self._total_count = self._count
             progress = int(self._count * self._bar_size / self._total_count)
             current_elapsed = time.time() - self._start_time
             remaining = 0 if self._count == 0 else ((self._total_count - self._count) *
@@ -925,7 +1011,7 @@ class Neo4jConnector(object):
                   u"#"*progress, "."*(self._bar_size-progress), self._count,
                   self._total_count, duration, rate, remaining), end=self._bar_end, flush=True)
 
-    class BasicArrowClientAuthHandler(pf.ClientAuthHandler):
+    class _BasicArrowClientAuthHandler(pf.ClientAuthHandler):
         def __init__(self, username, password):
             super().__init__()
             self.basic_auth = pf.BasicAuth(username, password)
@@ -962,10 +1048,16 @@ class Neo4jConnector(object):
             fields = ('nodeType', 'nodeLabels', 'propertyName', 'propertyTypes', 'mandatory')
             if self._neo4j_has_apoc:
                 q="CALL apoc.meta.nodeTypeProperties() YIELD nodeType, nodeLabels, propertyName, propertyTypes, mandatory RETURN *"
+                with self._neo4j_driver.query(q, False) as query:
+                    self._neo4j_node_type_properties = [{_ : record[_] for _ in fields} for record in query.result()]
+                # Get unlabeled nodes. APOC has a bug...
+                q="CALL db.schema.nodeTypeProperties() YIELD nodeType, nodeLabels, propertyName, propertyTypes, mandatory RETURN *"
+                with self._neo4j_driver.query(q, False) as query:
+                    self._neo4j_node_type_properties += [{_ : record[_] for _ in fields} for record in query.result() if record['nodeType'] == '']
             else:
                 q="CALL db.schema.nodeTypeProperties() YIELD nodeType, nodeLabels, propertyName, propertyTypes, mandatory RETURN *"
-            with self._neo4j_driver.query(q, False) as query:
-                self._neo4j_node_type_properties = [{_ : record[_] for _ in fields} for record in query.result()]
+                with self._neo4j_driver.query(q, False) as query:
+                    self._neo4j_node_type_properties = [{_ : record[_] for _ in fields} for record in query.result()]
         return self._neo4j_node_type_properties
 
     def __neo4j_rel_type_properties(self, flush_cache = True):
@@ -973,10 +1065,17 @@ class Neo4jConnector(object):
             fields = ('relType', 'propertyName', 'propertyTypes', 'mandatory')
             if self._neo4j_has_apoc:
                 q="CALL apoc.meta.relTypeProperties() YIELD relType, propertyName, propertyTypes, mandatory RETURN *"
+                with self._neo4j_driver.query(q, False) as query:
+                    self._neo4j_rel_type_properties = [{_ : record[_] for _ in fields} for record in query.result()]
+                # Get edges attached to unlabeled nodes. APOC has a bug...
+                found = {record['relType'] : True for record in self._neo4j_rel_type_properties }
+                q="CALL db.schema.relTypeProperties() YIELD relType, propertyName, propertyTypes, mandatory RETURN *"
+                with self._neo4j_driver.query(q, False) as query:
+                    self._neo4j_rel_type_properties += [{_ : record[_] for _ in fields} for record in query.result() if record['relType'] not in found]
             else:
                 q="CALL db.schema.relTypeProperties() YIELD relType, propertyName, propertyTypes, mandatory RETURN *"
-            with self._neo4j_driver.query(q, False) as query:
-                self._neo4j_rel_type_properties = [{_ : record[_] for _ in fields} for record in query.result()]
+                with self._neo4j_driver.query(q, False) as query:
+                    self._neo4j_rel_type_properties = [{_ : record[_] for _ in fields} for record in query.result()]
         return self._neo4j_rel_type_properties
 
     def __add_neo4j_schema_connectivity_to_neo4j_edges(self) -> None:
@@ -1014,7 +1113,6 @@ class Neo4jConnector(object):
 
                         if (not schema_exists):
                             continue
-
                     if self.__verbose:
                         print(f"Edge Connectivity: {e}")
                         print(f" -> type => {e_type}")
@@ -1026,10 +1124,46 @@ class Neo4jConnector(object):
                         self._neo4j_edges[e_type]['sources'] = set()
                         self._neo4j_edges[e_type]['targets'] = set()
                     self._neo4j_edges[e_type]['endpoints'].add(
-                        f"{extract_node_info(source)}->{extract_node_info(target)}")
+                        (extract_node_info(source), extract_node_info(target)))
                     self._neo4j_edges[e_type]['sources'].add(extract_node_info(source))
                     self._neo4j_edges[e_type]['targets'].add(extract_node_info(target))
 
+        vertices = [vertex for vertex in self.__neo4j_node_labels(False)]
+        for e_type in self._neo4j_edges:
+            if 'endpoints' not in self._neo4j_edges[e_type]:
+                # We have an edge with one or more non-typed vertex.
+                # These aren't picked up be Neo4j or APOC with the above methods.
+
+                self._neo4j_edges[e_type]['endpoints'] = set()
+                self._neo4j_edges[e_type]['sources'] = set()
+                self._neo4j_edges[e_type]['targets'] = set()
+
+                for v in vertices:
+                    if v == self.__neo4j_unlabeled_vertex_name():
+                        continue
+                    schema_exists = False
+                    q = f"MATCH (:{v})-[e:{e_type}]->() return count(e) > 0"
+                    with self._neo4j_driver.query(q, False) as query:
+                        for result in query.result():
+                            schema_exists = result[0]
+
+                    if (schema_exists):
+                        self._neo4j_edges[e_type]['sources'].add(v)
+                        self._neo4j_edges[e_type]['endpoints'].add((v, None))
+
+                    q = f"MATCH ()-[e:{e_type}]->(:{v}) return count(e) > 0"
+                    with self._neo4j_driver.query(q, False) as query:
+                        for result in query.result():
+                            schema_exists = result[0]
+
+                    if (schema_exists):
+                        self._neo4j_edges[e_type]['targets'].add(v)
+                        self._neo4j_edges[e_type]['endpoints'].add((None, v))
+
+            # Failed to find any relevant nodes for this edge.
+            if len(self._neo4j_edges[e_type]['endpoints']) == 0:
+                self._neo4j_edges[e_type]['endpoints'].add((None, None))
+
         return None
 
     def __neo4j_nodes(self, flush_cache = True):
@@ -1045,6 +1179,12 @@ class Neo4jConnector(object):
                         nodes[name] = dict()
                     if propTypes is not None:
                         nodes[name][prop['propertyName']] = propTypes
+                if len(labels) == 0:
+                    name = self.__neo4j_unlabeled_vertex_name()
+                    if name not in nodes:
+                        nodes[name] = dict()
+                    if propTypes is not None:
+                        nodes[name][prop['propertyName']] = propTypes
             self._neo4j_nodes = nodes
         return self._neo4j_nodes
 
@@ -1058,6 +1198,7 @@ class Neo4jConnector(object):
         for n in nodes:
             if self.__verbose:
                 print(f"__neo4j_process_nodes: {n}")
+
             propName = n['propertyName']
             propType = n['propertyTypes']
             if propType is not None and len(propType) == 1:
@@ -1067,6 +1208,12 @@ class Neo4jConnector(object):
                     res[name] = dict()
                 if propName is not None and propName not in res[name]:
                     res[name][propName] = propType
+            if len(n['nodeLabels']) == 0:
+                name = self.__neo4j_unlabeled_vertex_name()
+                if name not in res:
+                    res[name] = dict()
+                if propName is not None and propName not in res[name]:
+                    res[name][propName] = propType
         return res
 
     def __neo4j_process_edges(self, edges):
@@ -1087,9 +1234,9 @@ class Neo4jConnector(object):
                 res[relType]['schema'][propName] = propType
         return res
 
-    def __extract_xgt_vertex_schema(self, vertex, neo4j_id_name, flush_cache = True):
+    def __extract_xgt_vertex_schema(self, vertex, neo4j_id_name, xgt_vertex_name, flush_cache = True):
         if flush_cache:
-          self.__update_cache_state()
+            self.__update_cache_state()
         if vertex in self.__neo4j_nodes(False):
             neo4j_node = self.__neo4j_nodes(False)[vertex]
             neo4j_node_attributes = list(neo4j_node.keys())
@@ -1102,11 +1249,14 @@ class Neo4jConnector(object):
         schema.insert(0, [neo4j_id_name, xgt.INT])
         neo4j_schema.insert(0, [neo4j_id_name, 'INTEGER'])
         return {'schema' : schema, 'neo4j_schema' : neo4j_schema,
-                'key' : neo4j_id_name}
+                'key' : neo4j_id_name, 'xgt_name' : xgt_vertex_name}
 
     def __extract_xgt_edge_schema(self, edge, source, target,
                                   neo4j_source_node_name,
                                   neo4j_target_node_name,
+                                  xgt_edge_name,
+                                  xgt_source_name,
+                                  xgt_target_name,
                                   flush_cache = True):
         if flush_cache:
           self.__update_cache_state()
@@ -1118,12 +1268,17 @@ class Neo4jConnector(object):
             print(f"xGT graph schema for edge {edge}: {edge_info}")
         info_schema = edge_info['schema']
         edge_endpoints = edge_info['endpoints']
-        endpoints = f"{source}->{target}"
-        if endpoints not in edge_endpoints:
-            if len(edge_endpoints) == 1:
-                raise ValueError(f"Edge Type {edge} with endpoints: {endpoints} not found.")
-            else:
-                return
+        endpoints = (source, target)
+        if source != None and target != None and endpoints not in edge_endpoints:
+            raise ValueError(f"Edge Type {edge} with endpoints: {endpoints} not found.")
+        has_source = True
+        has_target = True
+        if source == None:
+            source = self.__neo4j_unlabeled_vertex_name()
+            has_source = False
+        if target == None:
+            target = self.__neo4j_unlabeled_vertex_name()
+            has_target = False
 
         schema = [[key, *self.__neo4j_type_to_xgt_type(type)]
                   for key, type in info_schema.items()]
@@ -1135,28 +1290,52 @@ class Neo4jConnector(object):
         return {'schema' : schema, 'neo4j_schema' : neo4j_schema,
                 'source' : source, 'target' : target,
                 'source_key' : neo4j_source_node_name,
-                'target_key' : neo4j_target_node_name}
+                'target_key' : neo4j_target_node_name,
+                'empty_labels' : self._Labels.get_label_type(has_source, has_target),
+                'xgt_name' : xgt_edge_name, 'xgt_source' : xgt_source_name,
+                'xgt_target' : xgt_target_name}
 
-    def __extract_xgt_edge_schemas(self, edge, vertices,
-                                    neo4j_source_node_name,
-                                    neo4j_target_node_name, flush_cache = True):
+    def __extract_xgt_edge_schemas(self, edge, vertices, edges, neo4j_source_node_name,
+                                   neo4j_target_node_name,
+                                   import_edge_nodes = True, flush_cache = True):
         if flush_cache:
           self.__update_cache_state()
         schemas = []
         neo4j_edge = self.__neo4j_edges(False)[edge]
-        if not 'sources' in neo4j_edge or not 'targets' in neo4j_edge:
-            raise ValueError(f"Untyped vertices not supported in {edge}")
-        for source in neo4j_edge['sources']:
-            if source not in vertices:
-                vertices[source] = None
-            for target in neo4j_edge['targets']:
-                if target not in vertices:
-                    vertices[target] = None
-                result = self.__extract_xgt_edge_schema(edge,
-                    source, target, neo4j_source_node_name,
-                    neo4j_target_node_name, flush_cache)
-                if result is not None:
-                    schemas.append(result)
+        for source, target in neo4j_edge['endpoints']:
+            source_name = source
+            target_name = target
+            if import_edge_nodes:
+                if source != None and source not in vertices:
+                    vertices[source] = source
+                if target != None and target not in vertices:
+                    vertices[target] = target
+            if source == None or target == None:
+                vname = self.__neo4j_unlabeled_vertex_name()
+                if source == None:
+                    source_name = vname
+                if target == None:
+                    target_name = vname
+                if import_edge_nodes:
+                    if not vname in vertices:
+                        vertices[vname] = self.__xgt_unlabeled_vertex_name()
+            if source_name in vertices:
+                source_name = vertices[source_name]
+            elif source_name == self.__neo4j_unlabeled_vertex_name():
+                source_name = self.__xgt_unlabeled_vertex_name()
+
+            if target_name in vertices:
+                target_name = vertices[target_name]
+            elif target_name == self.__neo4j_unlabeled_vertex_name():
+                target_name = self.__xgt_unlabeled_vertex_name()
+
+            result = self.__extract_xgt_edge_schema(edge,
+                source, target, neo4j_source_node_name,
+                neo4j_target_node_name, edges[edge], source_name,
+                target_name, flush_cache)
+            if result is not None:
+                schemas.append(result)
+
         return schemas
 
     def __neo4j_type_to_xgt_type(self, prop_type):
@@ -1169,7 +1348,7 @@ class Neo4jConnector(object):
 
     def __arrow_writer(self, frame_name, schema):
         arrow_conn = pf.FlightClient((self._xgt_server.host, self._xgt_server.port))
-        arrow_conn.authenticate(self.BasicArrowClientAuthHandler())
+        arrow_conn.authenticate(self._BasicArrowClientAuthHandler())
         writer, _ = arrow_conn.do_put(
             pf.FlightDescriptor.for_path(self._default_namespace, frame_name),
             schema)
@@ -1177,7 +1356,7 @@ class Neo4jConnector(object):
 
     def __arrow_reader(self, frame_name):
         arrow_conn = pf.FlightClient((self._xgt_server.host, self._xgt_server.port))
-        arrow_conn.authenticate(self.BasicArrowClientAuthHandler())
+        arrow_conn.authenticate(self._BasicArrowClientAuthHandler())
         return arrow_conn.do_get(pf.Ticket(self._default_namespace + '__' + frame_name))
 
     def __copy_data(self, cypher_for_extract, frame, neo4j_schema, progress_bar):
@@ -1339,6 +1518,12 @@ class Neo4jConnector(object):
         else:
             return edge
 
+    def __neo4j_unlabeled_vertex_name(self):
+        return ''
+
+    def __xgt_unlabeled_vertex_name(self):
+        return 'unlabeled'
+
     # TODO(landwehrj) : Is there a way to detect the cache is stale
     # One option is to use the Neo4j count store of relationship/nodes
     # to check if there are changes there. This wouldn't work in certain
