@@ -37,7 +37,6 @@ class TestXgtNeo4jConnector(unittest.TestCase):
   def setup_class(cls):
     # Create a connection to Trovares xGT
     cls.xgt = xgt.Connection()
-    cls.xgt.drop_namespace('test', force_drop = True)
     cls.xgt.set_default_namespace('test')
     cls.neo4j_driver, cls.conn, cls.conn_arrow = cls._setup_connector(cls.driver)
     return
@@ -179,11 +178,11 @@ class TestXgtNeo4jConnector(unittest.TestCase):
     self.neo4j_driver.query(
         'CREATE (:Node2{})-[:Relationship1{}]->(:Node1{}), (:Node1{})-[:Relationship2{int: 1}]->(:Node2{})').finalize()
     assert len(c.neo4j_edges) == 2
-    assert c.neo4j_edges['Relationship1']['endpoints'] == {'Node2->Node1'}
+    assert c.neo4j_edges['Relationship1']['endpoints'] == {('Node2', 'Node1')}
     assert c.neo4j_edges['Relationship1']['sources'] == {'Node2'}
     assert c.neo4j_edges['Relationship1']['targets'] == {'Node1'}
     assert c.neo4j_edges['Relationship1']['schema'] == {}
-    assert c.neo4j_edges['Relationship2']['endpoints'] == {'Node1->Node2'}
+    assert c.neo4j_edges['Relationship2']['endpoints'] == {('Node1', 'Node2')}
     assert c.neo4j_edges['Relationship2']['sources'] == {'Node1'}
     assert c.neo4j_edges['Relationship2']['targets'] == {'Node2'}
     assert c.neo4j_edges['Relationship2']['schema'] == {'int' : 'Long'}
@@ -193,7 +192,7 @@ class TestXgtNeo4jConnector(unittest.TestCase):
     self.neo4j_driver.query(
           'CREATE (:Node2{})-[:Relationship1{int: 2}]->(:Node1{}), (:Node1{})-[:Relationship1{int: 1}]->(:Node2{})').finalize()
     assert len(c.neo4j_edges) == 1
-    assert c.neo4j_edges['Relationship1']['endpoints'] == {'Node1->Node2', 'Node2->Node1'}
+    assert c.neo4j_edges['Relationship1']['endpoints'] == {('Node1', 'Node2'), ('Node2', 'Node1')}
     assert c.neo4j_edges['Relationship1']['sources'] == {'Node1', 'Node2'}
     assert c.neo4j_edges['Relationship1']['targets'] == {'Node1', 'Node2'}
     assert c.neo4j_edges['Relationship1']['schema'] == {'int' : 'Long'}
@@ -334,6 +333,19 @@ class TestXgtNeo4jConnector(unittest.TestCase):
     print(edge_frame.get_data())
     self.xgt.drop_frame("Relationship")
 
+  def test_transfer_relationship_working_types_bolt_map(self):
+    self._populate_relationship_working_types_bolt()
+    c = self.conn
+    xgt_schema = c.get_xgt_schemas(vertices=[('Node', 'n')], edges=[('Relationship', 'r')])
+    c.create_xgt_schemas(xgt_schema)
+    c.copy_data_to_xgt(xgt_schema)
+    node_frame = self.xgt.get_vertex_frame('n')
+    edge_frame = self.xgt.get_edge_frame('r')
+    assert node_frame.num_rows == 6
+    assert edge_frame.num_rows == 3
+    print(edge_frame.get_data())
+    self.xgt.drop_frame('r')
+
   def test_transfer_relationship_without_vertex_bolt(self):
     self._populate_relationship_working_types_bolt()
     c = self.conn
@@ -347,6 +359,20 @@ class TestXgtNeo4jConnector(unittest.TestCase):
     assert node_frame.get_data(length=1)[0][1] == 1
     self.xgt.drop_frame("Relationship")
 
+  def test_transfer_relationship_without_vertex_bolt_two_phase(self):
+    self._populate_relationship_working_types_bolt()
+    c = self.conn
+    c.transfer_to_xgt(vertices=['Node'])
+    xgt_schema = c.get_xgt_schemas(edges=['Relationship'], import_edge_nodes=False)
+    c.create_xgt_schemas(xgt_schema, append=True)
+    c.copy_data_to_xgt(xgt_schema)
+    node_frame = self.xgt.get_vertex_frame('Node')
+    edge_frame = self.xgt.get_edge_frame('Relationship')
+    assert node_frame.num_rows == 6
+    assert edge_frame.num_rows == 3
+    assert node_frame.get_data(length=1)[0][1] == 1
+    self.xgt.drop_frame("Relationship")
+
   def test_transfer_everything_bolt(self):
     self._populate_relationship_working_types_bolt()
     c = self.conn
@@ -500,6 +526,119 @@ class TestXgtNeo4jConnector(unittest.TestCase):
     with self.assertRaises(xgt.XgtNameError):
         c.transfer_to_neo4j(edges=['cars'])
 
+  def test_empty_labels(self):
+    c = self.conn
+    self.neo4j_driver.query('CREATE ()').finalize()
+    c.transfer_to_xgt(vertices=[''])
+    node_frame = self.xgt.get_vertex_frame('unlabeled')
+    assert node_frame.num_rows == 1
+
+  def test_map_empty_labels(self):
+    c = self.conn
+    self.neo4j_driver.query('CREATE ()').finalize()
+    c.transfer_to_xgt(vertices=[('', 'custom_name')])
+    node_frame = self.xgt.get_vertex_frame('custom_name')
+    assert node_frame.num_rows == 1
+    with self.assertRaises(xgt.XgtNameError):
+        node_frame = self.xgt.get_vertex_frame('unlabeled')
+
+  def test_empty_labels_schema(self):
+    c = self.conn
+    self.neo4j_driver.query('CREATE ({int:2})').finalize()
+    c.transfer_to_xgt(vertices=[''])
+    node_frame = self.xgt.get_vertex_frame('unlabeled')
+    res = self.xgt.run_job('match (v) return v.int')
+    assert node_frame.num_rows == 1
+    assert res.get_data()[0][0] == 2
+    self.neo4j_driver.query("MATCH (n) DETACH DELETE n").finalize()
+    self.neo4j_driver.query('CREATE ({str:"bbb"})').finalize()
+    c.transfer_to_xgt(vertices=[''])
+    node_frame = self.xgt.get_vertex_frame('unlabeled')
+    res = self.xgt.run_job('match (v) return v.str')
+    assert node_frame.num_rows == 1
+    assert res.get_data()[0][0] == "bbb"
+
+  def test_edge_empty_labels(self):
+    c = self.conn
+    self.neo4j_driver.query(
+        'CREATE ()-[:e1]->(), (:Node)-[:e2]->(), ()-[:e3]->(:Node)').finalize()
+    c.transfer_to_xgt(edges=['e1', 'e2', 'e3'])
+    node_frame = self.xgt.get_edge_frame('e1')
+    assert node_frame.num_rows == 1
+    node_frame = self.xgt.get_edge_frame('e2')
+    assert node_frame.num_rows == 1
+    node_frame = self.xgt.get_edge_frame('e3')
+    assert node_frame.num_rows == 1
+    node_frame = self.xgt.get_vertex_frame('unlabeled')
+    assert node_frame.num_rows == 4
+    node_frame = self.xgt.get_vertex_frame('Node')
+    assert node_frame.num_rows == 2
+
+  def test_edge_empty_labels_schema(self):
+    c = self.conn
+    self.neo4j_driver.query(
+        'CREATE ()-[:e1]->({int:1}), (:Node)-[:e2]->({str:"aaa"}), ({bool:True})-[:e3]->(:Node)').finalize()
+    c.transfer_to_xgt(edges=['e1', 'e2', 'e3'])
+    node_frame = self.xgt.get_edge_frame('e1')
+    res = self.xgt.run_job('match ()-[:e1]->(v) return v.int')
+    assert node_frame.num_rows == 1
+    assert res.get_data()[0][0] == 1
+    node_frame = self.xgt.get_edge_frame('e2')
+    res = self.xgt.run_job('match ()-[:e2]->(v) return v.str')
+    assert node_frame.num_rows == 1
+    assert res.get_data()[0][0] == "aaa"
+    node_frame = self.xgt.get_edge_frame('e3')
+    res = self.xgt.run_job('match (v)-[:e3]->() return v.bool')
+    assert node_frame.num_rows == 1
+    assert res.get_data()[0][0] == True
+    node_frame = self.xgt.get_vertex_frame('unlabeled')
+    assert node_frame.num_rows == 4
+    node_frame = self.xgt.get_vertex_frame('Node')
+    assert node_frame.num_rows == 2
+
+  def test_map_edge_empty_labels(self):
+    c = self.conn
+    self.neo4j_driver.query(
+        'CREATE ()-[:e1]->(), (:Node)-[:e2]->(), ()-[:e3]->(:Node)').finalize()
+    c.transfer_to_xgt(vertices=[('', 'custom_empty'), ('Node', 'custom_Node')], edges=[('e1', 'edge1'), ('e2', 'edge2'), 'e3'])
+    node_frame = self.xgt.get_edge_frame('edge1')
+    assert node_frame.num_rows == 1
+    node_frame = self.xgt.get_edge_frame('edge2')
+    assert node_frame.num_rows == 1
+    node_frame = self.xgt.get_edge_frame('e3')
+    assert node_frame.num_rows == 1
+    node_frame = self.xgt.get_vertex_frame('custom_empty')
+    assert node_frame.num_rows == 4
+    node_frame = self.xgt.get_vertex_frame('custom_Node')
+    assert node_frame.num_rows == 2
+    with self.assertRaises(xgt.XgtNameError):
+        node_frame = self.xgt.get_vertex_frame('unlabeled')
+    with self.assertRaises(xgt.XgtNameError):
+        node_frame = self.xgt.get_vertex_frame('Node')
+
+  def test_edge_empty_labels_schema_no_implicit(self):
+    c = self.conn
+    self.neo4j_driver.query(
+        'CREATE ()-[:e1]->({int:1}), (:Node)-[:e2]->({str:"aaa"}), ({bool:True})-[:e3]->(:Node)').finalize()
+    c.transfer_to_xgt(vertices=['', 'Node'])
+    c.transfer_to_xgt(edges=['e1', 'e2', 'e3'], append=True, import_edge_nodes=False)
+    node_frame = self.xgt.get_edge_frame('e1')
+    res = self.xgt.run_job('match ()-[:e1]->(v) return v.int')
+    assert node_frame.num_rows == 1
+    assert res.get_data()[0][0] == 1
+    node_frame = self.xgt.get_edge_frame('e2')
+    res = self.xgt.run_job('match ()-[:e2]->(v) return v.str')
+    assert node_frame.num_rows == 1
+    assert res.get_data()[0][0] == "aaa"
+    node_frame = self.xgt.get_edge_frame('e3')
+    res = self.xgt.run_job('match (v)-[:e3]->() return v.bool')
+    assert node_frame.num_rows == 1
+    assert res.get_data()[0][0] == True
+    node_frame = self.xgt.get_vertex_frame('unlabeled')
+    assert node_frame.num_rows == 4
+    node_frame = self.xgt.get_vertex_frame('Node')
+    assert node_frame.num_rows == 2
+
   def _populate_node(self):
     self.neo4j_driver.query(
       # Integer, Float, String, Boolean, Point, Date, Time, LocalTime,
@@ -589,3 +728,4 @@ class TestXgtNeo4jConnector(unittest.TestCase):
 
   def _erase_neo4j_database(self):
     self.neo4j_driver.query("MATCH (n) DETACH DELETE n").finalize()
+    self.xgt.drop_namespace('test', force_drop = True)
